using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.SharePoint; // Added for SharePoint
using System.Net.Mail;      // Added for Mail access

namespace DocStatusEventHandler
{
    public class DocStatusChk : SPItemEventReceiver
    {
        public override void ItemDeleting(SPItemEventProperties properties)
        {
            // Do not permit deletions:
            properties.Cancel = true;
            // Return an Error message that will be shown to the user:
            string ErrMsg = "Deleting Items from ";
            ErrMsg += properties.ListTitle.ToString();
            ErrMsg += " is not permitted.";
            properties.ErrorMessage = ErrMsg;
        }

        public override void ItemUpdated(SPItemEventProperties properties)
        {
            //
            // Open the item to check the status of the MailSent field;
            // this prevents us from updating more than once:
            //
            bool WasMailSent = false;
            //
            // Try catch just in case field not found:
            //
            try
            {
                WasMailSent = (bool)properties.ListItem["MailSent"];
            }
            catch (Exception FldNotFound)
            {
                return;
            }
            //
            if (WasMailSent)
            {
                // Sent - do nothing
                return;
            }
            //
            // Check status of the item to determine if final:
            //
            string DocumentStatus = "";
            //
            // Try catch just in case field not found:
            //
            try
            {
                DocumentStatus = (string)properties.ListItem["Status"];
            }
            catch (Exception FldNotFound)
            {
                return;
            }
            //
            if (DocumentStatus != "Final")
            {
                // Not a status we are concerned with
                return;
            }
            //
            // Status is final, send email to the author:
            //
            SPWeb TheCurWeb = properties.ListItem.ParentList.ParentWeb;
            // Get the author (this field is in all List types):
            string Author = (string)properties.ListItem["Author"];
            // Get the ID:
            int IDOf = Convert.ToInt32(Author.Substring(0, Author.IndexOf(";")));
            SPUser CreatedBy = TheCurWeb.SiteUsers.GetByID(IDOf);
            // Set the To address:
            string ToWhom = CreatedBy.Email;
            //
            string SmtpServerName = "MOSSSPSDEV";    // or server name
            string FromAddress = "spadmin@mosswss.com";
            //
            SmtpClient client = new SmtpClient(SmtpServerName);
            // Set From address:
            MailAddress from = new MailAddress(FromAddress, "SharePoint Admin", System.Text.Encoding.UTF8);
            // Set recipient for the e-mail message:
            MailAddress to = new MailAddress(ToWhom);
            // Specify the message content:
            MailMessage message = new MailMessage(from, to);
            //
            message.IsBodyHtml = true;
            message.BodyEncoding = System.Text.Encoding.UTF8;
            message.SubjectEncoding = System.Text.Encoding.UTF8;
            //
            message.Subject = "Document has been finalized";
            //
            // Start the message:
            //
            string MessageBody = "Your document: <br>";
            //
            // Use the web url to construct a link to the document:
            //
            string UrlOf = properties.WebUrl;
            if (UrlOf.Substring(UrlOf.Length - 1) != "/")
            {
                UrlOf = UrlOf + "/";
            }
            string ItemUrl = UrlOf + "Lists/" 
                + properties.ListTitle
                + "/DispForm.aspx?ID=" 
                + properties.ListItem.ID.ToString();
            MessageBody += "<a href=\"" 
                + ItemUrl + "\">" 
                + properties.ListItem.Name 
                + "</a><br><br>";
            MessageBody += "was finalized at " + System.DateTime.Now.ToString();
            message.Body = MessageBody;
            //
            // Send the message:
            //
            try
            {
                client.Send(message);
            }
            catch (Exception ErrInSend)
            {
                string WhyErr = ErrInSend.Message.ToString();
            }
            //
            // Now since the message has been sent, set the flag to
            // prevent us from doing this again.
            //
            // Disable event firing while we update:
            //
            base.DisableEventFiring();
            //
            properties.ListItem["MailSent"] = true;
            properties.ListItem.Update();
            //
            base.EnableEventFiring();
            //

        }
    }
}
