using System;
using System.Runtime.InteropServices;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Serialization;

using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.WebPartPages;

using System.Web.UI.WebControls; // Added for controls
using System.Drawing;            // Added for color, add a reference too
using System.Text;               // Added for StringBuilder
using System.Resources;          // Added for Resources

namespace HowManyLists
{
    [Guid("3c478a67-d437-4e8a-b2eb-2650ad26088b")]
    public class HMLClass : System.Web.UI.WebControls.WebParts.WebPart
    {
        // Save for times checked:
        //
        public int HitCount;
        public string OurDisplayTitle = "Get List Count";
        //
        //==================================================================
        // Define Child Controls
        //
        protected Label DisplayCount = new Label();
        protected Button CalcListCount = new Button();
        //
        //==================================================================
        // PROPERTY: ShowHitCount
        // This is a settable flag that turns the Hit Count display
        // On or Off (Checked = True or On). Note that we set the default
        // as 'true' (we will override this in the webpart file).
        // Also notice the XML settings - these set the web part property
        // category to "Custom Settings" (this is what displays when
        // the web part properties are displayed), sets the storage to be
        // Shared (setting applies to all users), sets personalization to 
        // be shared and is browsable (visible) in the web part properties.
        // 
        private const bool defaultShowHitCount = false;
        private bool showHitCount = defaultShowHitCount;
        [SPWebCategoryName("Custom Settings"),
          WebPartStorage(Storage.Shared),
          WebBrowsable(true),
          Personalizable(PersonalizationScope.Shared),
          WebDescription("Sets true/false to show hit count"),
         WebDisplayName("Check to show hit count")]
        public bool ShowHitCount
        {
            get { return showHitCount; }
            set { showHitCount = value; }
        }
        //
        //==================================================================
        // METHODS: 
        //==================================================================
        // The following are the various methods as executed in the order 
        // shown (this is EXACTLY the same as a Web Control):
        //==================================================================
        // ONINIT EVENT:
        // Fires 1st: Startup of the Control and where in web parts, the
        // storage system is loaded; also calls LoadViewState (loads the
        // view state for all controls); if you uncomment this event, you
        // MUST include the base.OnInit(e) line
        //
        //protected override void OnInit(System.EventArgs e)
        //{
        //        base.OnInit(e); // You must include this if you override
        //}
        //
        //==================================================================
        // ONLOAD EVENT:
        // Fires 2nd: OnLoad event - this is where startup functionality/
        // processing happens, including trapping PostBack - AFTER THE
        // OnLoad events, any User Generated events (Click, etc.) are fired
        //
        protected override void OnLoad(System.EventArgs e)
        {
            base.OnLoad(e);
            //
            // This is the area to establish DB connections and
            // data retrieval
            //
            if (Page.IsPostBack)
            {
                // PostBack processing:
                try
                {
                    string HCountString = (string)this.ViewState["HITCOUNT"];
                    if (HCountString != "" || HCountString == null)
                    {
                        HitCount = Convert.ToInt32(HCountString);
                        HitCount++;
                        this.ViewState["HITCOUNT"] = HitCount.ToString();
                    }
                }
                catch
                {
                    this.ViewState["HITCOUNT"] = "0";
                    HitCount = 0;
                }
                // Save it either way:
                this.SaveViewState();
            }
        }
        //
        //==================================================================
        // CREATECHILDCONTROLS EVENT:
        // Fires 3rd (BEFORE any events and OnPreRender)- this creates any
        // controls; note that things like drop downs, etc. have to either 
        // maintain state or be reloaded here
        //
        protected override void CreateChildControls()
        {
            //
            // Create the label:
            //
            DisplayCount.ID = "DisplayCount";
            DisplayCount.Text = "";
            DisplayCount.ForeColor = System.Drawing.Color.Blue;
            DisplayCount.Font.Bold = true;
            // VERY Important: Be sure to add the control:
            Controls.Add(DisplayCount);
            //
            // Create the button:
            //
            CalcListCount.Click += new EventHandler(CalcListCount_Click);
            CalcListCount.ID = "CalcListCount";
            CalcListCount.Text = "Click Here";
            CalcListCount.Font.Size = FontUnit.XXSmall;
            CalcListCount.Font.Bold = true;
            CalcListCount.CausesValidation = true;
            Controls.Add(CalcListCount);
            //
        }
        //
        //==================================================================
        // EVENT HANDLERS:
        // Fires 4th: Event Handlers - event handlers fire just AFTER 
        // CreateChildControls and before OnPreRender:
        //
        void CalcListCount_Click(object sender, EventArgs e)
        {
            //
            // On click, we need to calculate the current number of lists
            // in the current Site:
            //
            // Get a ref to the current site:
            //
            SPWeb CurrentSite = SPControl.GetContextWeb(Context);
            SPListCollection SPLC = CurrentSite.Lists;
            //
            // Show the value:
            //
            DisplayCount.Text = "Found: " + SPLC.Count.ToString() + " lists";
            //
        }
        //
        //==================================================================
        // ONPRERENDER EVENT: 
        // Fires 5th: OnPreRender event - called just before anything is
        // drawn via the Render event (here is where Properties can be
        // changed before the controls are drawn); this event calls
        // SaveViewState to save the view state before rendering
        //
        //protected override void OnPreRender(System.EventArgs e)
        //{
        //        base.OnPreRender(e);
        //}
        //
        //==================================================================
        // RENDER EVENT: 
        // Fires 6th (last): Render event - Render this control to the
        // output (HTML Writer) parameter specified
        // <param name="output">The HTML writer to write out to</param>
        // 
        protected override void Render(HtmlTextWriter writer)
        {
            //
            // Make sure Child Controls have been setup:
            //
            EnsureChildControls();
            //
            //--------------------------------------------------------------
            // Best practices tip: When writing out, ALWAYS use a table;
            // this prevents getting out of whack when placed on the page
            //--------------------------------------------------------------
            // Get the title from the Resources:
            //
            string TheButtonText = "Get List Count";
            //
            ResourceManager WPResourceManager =
                new ResourceManager("HowManyLists.HMLResources", GetType().Assembly);
            //
            try
            {
                string TestButtonText = WPResourceManager.GetString("ButtonText");
                if (TestButtonText.Length > 0)
                {
                    TheButtonText = TestButtonText;
                }
            }
            catch (System.Resources.MissingManifestResourceException ErrInName)
            {
                // Means a problem with the name
                string MMREText = ErrInName.Message.ToString();
            }
            catch (Exception ErrInResource)
            {
                string SomeOtherProbText = ErrInResource.Message.ToString();
            }
            WPResourceManager.ReleaseAllResources();
            //
            // Assign it to the control:
            //
            CalcListCount.Text = TheButtonText;
            //
            //--------------------------------------------------------------
            // You can either use a string to build the display or simply
            // write it out as you go - I suggest using a string since
            // it will speed up rendering
            //
            StringBuilder OSPart1 = new StringBuilder();
            StringBuilder OSPart2 = new StringBuilder();
            StringBuilder OSPart3 = new StringBuilder();
            //
            // Build the output strings:
            //
            // NOTE: HTML attributes should be enclosed in quotes,
            // for example:
            //      "<table width=\"200\"
            // however I have left them out for readability
            //
            OSPart1.Append("<table width=200 cellspacing=0 border=0>");
            OSPart1.Append("<tr width=100% bgcolor=white>");
            // Notice that I've used a standard WSS Style:
            OSPart1.Append("<td colspan=2 width=100% class='ms-wptitle'>");
            OSPart1.Append(OurDisplayTitle);
            OSPart1.Append("</td></tr>");
            OSPart1.Append("<tr width=100% bgcolor=white>");
            OSPart1.Append("<td width=100% align=center class='ms-ls'>");
            //
            OSPart2.Append("</td></tr>");
            try
            {
                if (HitCount > 0 && ShowHitCount)
                {
                    string DispHits = HitCount.ToString();
                    OSPart2.Append("<tr width=100% bgcolor=white>");
                    OSPart2.Append("<td width=100% align=center class='ms-ls'>");
                    OSPart2.Append("You've checked: " + DispHits + " times");
                    OSPart2.Append("</td></tr>");
                }
            }
            catch { }
            OSPart2.Append("<tr width=100%>");
            OSPart2.Append("<td width=100% align=center valign=center>");
            //
            OSPart3.Append("</td></tr>");
            OSPart3.Append("</table>");
            //
            // Render:
            //
            writer.Write(OSPart1);
            if (DisplayCount.Text == "" || DisplayCount.Text == null)
            {
                writer.Write("<br><br>");
            }
            else
            {
                DisplayCount.RenderControl(writer);
                if (!ShowHitCount) // Fill in space
                {
                    writer.Write("<br><br>");
                }
            }
            writer.Write(OSPart2);
            CalcListCount.RenderControl(writer);
            writer.Write(OSPart3);
        }
    }
}